from os.path import exists, join
from BeautifulSoup import BeautifulSoup
from urllib import urlopen


class ReportFilesNotFound(Exception):
    def __init__(self, reason):
        self.message = reason


class JenkinsDevelReport:
    ROOT_URL = "https://lpci.wedontsleep.org/job/devel/"
    def __init__(self):
        self._last_deployed_revno = None
        self._last_build = None
        self.bs_last_successful_build = None
        self.bs_last_build = None

    def get_beautiful_index(self, url):
        page = urlopen(url)
        lines = "".join(page.readlines())
        return BeautifulSoup(lines)

    def get_beautiful_last_successful_build_changes(self):
        url = self.ROOT_URL + "lastSuccessfulBuild/changes"
        return self.get_beautiful_index(url)

    def get_beautiful_last_successful_build(self):
        url = self.ROOT_URL + "lastSuccessfulBuild"
        return self.get_beautiful_index(url)

    def get_beautiful_last_build(self):
        url = self.ROOT_URL + "lastBuild"
        return self.get_beautiful_index(url)

    @property
    def last_successful_build(self):
        if not self._last_deployed_revno:
            # get bs part
            if not self.bs_last_successful_build:
                self.bs_last_successful_build_changes = (
                    self.get_beautiful_last_successful_build_changes())
                self.bs_last_successful_build = (
                    self.get_beautiful_last_successful_build())

            bs = self.bs_last_successful_build_changes
            revno = int(bs.find(name="div", attrs={"class":"changeset-message"}
                ).find(name="a").contents[0].split()[0])

            bs = self.bs_last_successful_build
            build_text = bs.find(name="h1").text.split("\n")
            self.last_successful_buildno = build_text[0].split("#")[1]

            if revno:
                self._last_deployed_revno = revno

        return self._last_deployed_revno, self.last_successful_buildno

    @property
    def last_build(self):
        if not self._last_build:
            if not self.bs_last_build:
                self.bs_last_build = self.get_beautiful_last_build()
            bs = self.bs_last_build
            self.current_elapsed_time = bs.find(name="div",
                attrs={"style":"margin-top:1em"}).contents[0]
            build_text = bs.find(name="h1").text.split("\n")
            buildno = build_text[0].split("#")[1]
            if buildno:
                self._last_build = buildno
        return self._last_build, self.current_elapsed_time


### Parsing deployment reports

class DeploymentReport:

    def __init__(self, reports_folder, report_name):
        self.report_name = report_name
        self.short_name = report_name.strip('.html').replace('deployment-', '')
        self.report_path = join(reports_folder, report_name)
        if not exists(self.report_path):
            raise ReportFilesNotFound(
                "Couldn't find deployment report to load.")

        self.fixed_bugs = []
        self.contents = None

    def fetch_contents(self):
        self.contents = "".join(open(self.report_path).readlines())
        self.bs = BeautifulSoup(self.contents)

    def get_deployment_qa_data(self):
        if not self.contents:
            self.fetch_contents()
        # Get current deployed to qa revno and current number of revnos
        # blocked in QA
        revnos = self.bs.find(name="section",
            attrs={"id":"pending_revisions"}).findAll(name="strong")
        if len(revnos) > 1:
            self.current_deployed_to_qa_revno, self.blocked_revnos_count = revnos
            self.blocked_revnos_count = int(self.blocked_revnos_count.string)
        else:
            self.current_deployed_to_qa_revno = revnos[0]
            self.blocked_revnos_count = 0
        self.current_deployed_to_qa_revno = int(
            self.current_deployed_to_qa_revno.string)

        # Get current blocking revno
        try:
            self.current_blocker_details = {}
            self.current_blocking_revno = int(self.bs.find(name="section",
                attrs={"id":"blocker"}).find(name="strong").string.replace(
                    "Revision ", ""))
            current_blocker_details = self.bs.find(name="table",
                attrs={"id":"blocker_detail"}).find(name="td")
            self.current_blocker_details["bug"] = current_blocker_details.find(
                name="a").contents[0].strip().split()[1]
            self.current_blocker_details["reason"] = current_blocker_details.find(
                name="strong").string
        except:
            self.current_blocking_revno = None
            self.current_blocker_details = None

        # Get current number of revnos ready to deploy (current deployed +
        #   current ready to deploy)
        try:
            self.current_deployable_revno = int(self.bs.find(name="section",
                attrs={"id":"deployable_revision"}).find(
                name="strong").string.replace("Revision ", ""))
        except:
            self.current_deployable_revno = None
            self.current_ready_to_deploy = 0

        try:
            all_deployable_revnos = self.bs.find(name="section",
                attrs={"id":"deployments"}).findAll(name="li",
                attrs={"class":"deployable"})
        except AttributeError:
            all_deployable_revnos = []

        fixed_bugs = []
        first_deployable_revno = None
        for i in all_deployable_revnos:
            try:
                if not first_deployable_revno:
                    first_deployable_revno = int(
                        i.find(name="a").contents[0].split()[1])
                fixed_bugs.append(i.find(name="div",
                    attrs={"class":"buglinks"}).find(
                    name="a").contents[0].replace("Bug:", ""))
            except:
                continue

        self.fixed_bugs = fixed_bugs
        if all_deployable_revnos:
            self.current_deployed_revno = first_deployable_revno - 1
        else:
            # XXX: if first deployable revno is none, so the first blocked is
            # the one.
            if self.current_blocking_revno:
                self.current_deployed_revno = self.current_blocking_revno - 1
            else:
                self.current_deployed_revno = self.current_deployed_to_qa_revno
        if self.current_deployable_revno:
            self.current_ready_to_deploy = (
                self.current_deployable_revno - self.current_deployed_revno)
        else:
            self.current_ready_to_deploy = 0

    @property
    def deployable_bugs(self):
        return self.fixed_bugs

