import re
from bzrlib.transport import get_transport
from bzrlib.errors import InvalidHttpResponse
from BeautifulSoup import BeautifulSoup
from os.path import exists
from urllib import urlopen
from devops.utils import last_date_delta

__metaclass__ = type


class Instance:
    """Provide information about the Launchpad instances."""

    def __init__(self, name, url):
        self.name = name
        self.url = url
        self.is_responding = True
        self.is_ready = True
        self.last_deployed_revno = None
        self.hard_timeout_limit = None

    def get_index(self):
        """Get instance info."""
        try:
            transport = get_transport(self.url)
            html = transport.get_bytes('index.html')
        except InvalidHttpResponse, e:
            self.is_responding = True
            self.is_ready = False
        except:
            self.is_responding = True
            self.is_ready = False

        if self.is_ready:
            soup = BeautifulSoup(html)

            lp_version_comment = soup.find(
                text=re.compile("external actions issued in"))
            revision_re = re.compile(r'r(\d+)')
            matches = revision_re.search(lp_version_comment)
            if matches:
                self.last_deployed_revno = int(matches.group(1))
            hard_timeout_re = re.compile(r'(\d+)')
            matches = hard_timeout_re.search(lp_version_comment.split(
                "hard_timeout")[1])
            if matches:
                self.hard_timeout_limit = int(matches.group(1))

    def get_info(self):
        self.get_index()


class Production(Instance):
    def __init__(self):
        super(Production, self).__init__(
            "production", "https://launchpad.net/")


class QAStaging(Instance):
    def __init__(self):
        super(QAStaging, self).__init__(
            "qastaging", "https://qastaging.launchpad.net/")


class Staging(Instance):

    def __init__(self):
        self.last_db_update = {
            'date'  : "",
            'time' : "",
            'revno'  : "",
            'time_ago': "",
        }
        self.last_code_update = {
            'date'  : "",
            'time' : "",
            'revno'  : "",
            'time_ago': "",
        }
        super(Staging, self).__init__(
            "staging", "https://staging.launchpad.net/")

    def get_last_updates_info(self, filename="successful-updates.txt"):
        """Find last information about staging updates."""
        last_code_update = ""
        last_db_update = ""

        if not exists(filename):
            staging_update_entries = urlopen(
                "https://staging.launchpad.net/successful-updates.txt").readlines()
        else:
            staging_update_entries = open(filename).readlines()

        staging_update_entries.reverse()
        for i in staging_update_entries:
            if "Code update" in i:
                if last_code_update == "":
                    last_code_update = i.strip()
            if "Full update" in i:
                last_db_update = i.strip()
                if last_code_update == "":
                    last_code_update = i.strip() 
                break

        # Parse the entry
        line = last_code_update.split(" ")
        # 0: date
        # 1: time (UTC?)
        # n-1: revno
        date_ = line[0].replace(":", "-")
        self.last_code_update = {
            'date' : date_,
            'time' : line[1],
            'revno': line[-1],
            'time_ago': last_date_delta(date_),
        }
        if not self.last_deployed_revno:
            self.last_deployed_revno = self.last_code_update['revno']
        # Parse the entry
        line = last_db_update.split(" ")
        date_ = line[0].replace(":", "-")
        self.last_db_update = {
            'date' : date_,
            'time' : line[1],
            'revno': line[-1],
            'time_ago': last_date_delta(date_),
        }

    def get_info(self):
        self.get_index()
        self.get_last_updates_info()

