import unittest
from lplib_fakes.fakes import FakeLP, FakeBug

from devops.fetchbugs import LaunchpadBugs


class TestFetchBugs(unittest.TestCase):

    def setUp(self):
        self.fake_lp = FakeLP()
        self.fake_lp.createProjectGroup("launchpad-project")

        self.fake_lp.createBugWithBugTask("123456", "launchpad",
            u"Fix Committed", u"Critical", "Make foo bar")

        self.lp_bugs = LaunchpadBugs(lp=self.fake_lp)
        self.link_bugs("launchpad-project")

    def link_bugs(self, project):
        # This is to make the project group aware of the bugs in lp_bugs, so I
        # can use the same interface lplib has to searchTasks.
        self.fake_lp.project_groups[project].bug_tasks = self.fake_lp.bug_tasks

    def test_bugtask_is_returned(self):
        self.assertEqual([self.fake_lp.bugs["123456"].bug_tasks[0]],
            self.lp_bugs.fetch_critical_bugs())

    def test_no_critical_bugtask_found(self):
        self.fake_lp.bugs["123456"].bug_tasks[0].importance = u"High"
        self.assertEqual([], self.lp_bugs.fetch_critical_bugs())

    def test_open_critical_bugs_count(self):
        self.fake_lp.createBugWithBugTask("567890", "launchpad",
            u"Fix Committed", u"High", "Make foo bar again")
        self.assertEqual(len(self.lp_bugs.fetch_open_critical_bugs()), 1)
        self.fake_lp.bugs["567890"].bug_tasks[0].importance = u"Critical"
        self.assertEqual(len(self.lp_bugs.fetch_open_critical_bugs()), 2)

    def test_all_critical_bugs_count(self):
        self.fake_lp.createBugWithBugTask("567890", "launchpad",
            u"Fix Committed", u"Critical", "Make foo bar again")
        self.fake_lp.createBugWithBugTask("123123", "launchpad",
            u"Fix Released", u"Critical", "Foobarizing the bla")
        self.fake_lp.createBugWithBugTask("666666", "launchpad",
            u"Confirmed", u"Critical", "Exorcise me")
        self.fake_lp.createBugWithBugTask("111111", "launchpad",
            u"New", u"Undecided", "I'm a palindrome")

        self.link_bugs("launchpad-project")

        self.assertEqual(len(self.lp_bugs.fetch_critical_bugs()), 4)
        self.assertEqual(len(self.lp_bugs.fetch_open_critical_bugs()), 3)
