from datetime import datetime
from os.path import dirname, join
from unittest import TestCase
from urlparse import urljoin

from devops import incidentreports
from devops.incidentreports import IncidentReports


class TestIncidentReports(TestCase):


    def setUp(self):
        source_url = join(
            dirname(__file__), "sample_data", "incident-reports.html")
        self.ir = IncidentReports(source_url=source_url)

    def test_get_last_reports_since(self):
        incidentreports._today = datetime(2011, 11, 9)
        self.ir.get_last_reports_since()
        self.assertEqual(self.ir.total_incidents, 7)
        self.assertEqual(self.ir.most_days_with_no_incidents, 89)
        self.assertEqual(self.ir.last_report, '2011-11-07-LP-some-error')

    def test_no_incidents_for_a_long_time(self):
        # Test when the last incident report is older than the delta between
        # incident reports. Bug 929538
        incidentreports._today = datetime(2012, 2, 9)
        self.ir.get_last_reports_since()
        self.assertEqual(self.ir.most_days_with_no_incidents, 94)
        self.assertEqual(self.ir.last_report, '2011-11-07-LP-some-error')
